function biterror = OFDMreceiver()

clear all;
load OFDMspace.mat;
load MAX_SN.mat;

B = B;                      %Bandwidth
N = N;                      %Number of carriers               
Tu = Tu;                    %Symbol length
FS=FS;                      %Sampling freq
T = 1/FS;                   %Elementary period
fc = fc;                    %Carrier freq
tt = 0:T:Tu;                %Time vector
M = M;                      %Number of symbols - reference symbol
f = (1:FS);                 %Frequency vector
ttotal= 0:T:Tu*(M+1)+M*T;   %Total signal time

%Reference vector
Data_par = Data_par;

%RECEPTION
max_sn = MAX_SN;
% ref_normalized = REF;

[m_normalized d] = wavread('signal');
m = max_sn.*m_normalized;
% ref = max_sn.*ref_normalized;

%Removing delay
m_sync = zeros(length(m_normalized),1);
m_short = m(1:400000);
x = xcorr(ref, m_short);
[C I] = max(x);
delay = length(m_short)-I;

if delay < 0
    delay = 0;
end

m_sync(1:(length(m)-delay),1) = m((delay+1):(length(m)));

m_no_guard = zeros(1, length(ttotal));

active = 0;
for symbreceived = 1:M+1;
%Removing guard
m_no_guard = m_sync((1+(length(tt)*(symbreceived-1))+guard_samples*(symbreceived-1)*active):length(tt)*symbreceived+guard_samples*(symbreceived-1)*active);

if symbreceived == 1
    ref_chan = m_no_guard;
end

%Downshifting
r_tilde = m_no_guard.*exp(-i*2*pi*fc*tt'); 

%Lowpass filtering
[bb aa] = butter(13, 1/2);

r = filter(bb, aa, r_tilde);

%OFDM
info(1:length(r),symbreceived) = (1/FS).*fft(r);

%Removing Zero padding
info_1(1:(N/2),symbreceived) = info(1:freq_space_factor:(freq_space_factor*(N/2)), symbreceived);
info_1((N/2+1):N, symbreceived) = info((length(tt)-(freq_space_factor*(N/2)-1)):freq_space_factor:length(tt), symbreceived);
active = 1;
end

info_1_dim = size(info_1);
data = zeros(N, M); 
phase_shift_2 = zeros(N, M);

for rr = 2:info_1_dim(2);
    for ss = 1:info_1_dim(1);

    %Finding the phase difference
    phase_shift_2(ss,rr-1) = info_1(ss,rr)*conj(info_1(ss, rr-1));
    
    %Demodulating the differential coding
       if abs(imag(phase_shift_2(ss,rr-1)))>abs(real(phase_shift_2(ss,rr-1)));
            if imag(phase_shift_2(ss,rr-1))<0;     %-i
                data(ss,rr-1) = 1-i;   
            elseif imag(phase_shift_2(ss,rr-1))>0; %i
                data(ss,rr-1) = -1+i;  
            end
        elseif abs(imag(phase_shift_2(ss,rr-1)))<abs(real(phase_shift_2(ss,rr-1)));
            if real(phase_shift_2(ss,rr-1))<0;     %-1
                data(ss,rr-1) = 1+i;   
            elseif real(phase_shift_2(ss,rr-1))>0; %1  
                data(ss,rr-1) = -1-i; 
            end
       end 
    end

end

%Finding the QPSK errors
Ulike_par = 0;
 for k = 1:M
     for l = 1:N
        if data(l,k)==Data_par(l,k)
        Ulike_par = Ulike_par;
        else
        Ulike_par = Ulike_par + 1;
        end
     end
end

DataIn = DataIn;
DataOut = QPSKdemodulator(data, N, M);

%If data_type is sound, a wav-file is created
if strcmp(data_type, 'sound');
   wavData = writeSound(DataOut, Fs_sound, bits, plots);
end

%Comparing the DataOut vector with DataIn vector
Ulike_bit = 0; 
for j = 1:length(DataOut);
    if DataOut(j)==DataIn(j);
        Ulike_bit = Ulike_bit;
    else
        Ulike_bit = Ulike_bit + 1;
    end
end

clc;
%display in MATLAB
disp(['OFDM PARAMETERS:'])
disp(['Data type:                    ',data_type])
disp(['Symbol length:                ',num2str(Tu),' s'])
disp(['Number of carriers:           ',num2str(N)])
disp(['Number of symbols:            ',num2str(M)])
disp(['Guard time:                   ',num2str(guard),' s'])
disp(['   '])

disp(['CHANNEL PARAMETERS:'])
disp(['Signal to noise ratio:        ' num2str(SNR) ' dB'])
disp(['Phase error:                  ' num2str(phase_error) ' rad'])
disp(['Impulseresponse:              ' num2str(impulseresponse) ' : 1 Enabled, 0 disabled'])
disp(['Delay:                        ' num2str(channel_delay) ' s'])
disp(['   '])

biterror = Ulike_bit/length(DataOut);
disp(['RESULTS:'])
disp(['Number of QPSK-symbol errors: ' num2str(Ulike_par) '/' num2str(M*N)])
disp(['Number of bit errors:         ' num2str(Ulike_bit) '/' num2str(length(DataOut))])
disp(['BER:                          ' num2str(biterror)])
disp(['   '])


save('ref_chan.mat', 'ref_chan')

if plots == 1
    
%Plot of the baseband spectrum

figure();
subplot(211);
plot(f/1000, abs(fft(r, FS))/FS);
title('Spectrum of the received signal r(t)');
axis([0 192 0 1.5]);
xlabel('Frequency - kHz');
ylabel('Amplitude');
subplot(212);
pwelch(r,[],[],[],FS);

%Plot of the received phase shifts before decision
figure();
% plot(phase_shift_2(1,1:2),'LineStyle','none', 'Marker','x', 'MarkerSize',10);
plot(phase_shift_2,'LineStyle','none', 'Marker','x', 'MarkerSize',10);
title(['Phase difference - SNR ', num2str(SNR), ' dB']);
axis([-0.5 0.5 -0.5 0.5]);
grid on;
xlabel('Real');
ylabel('Imag');

%Plot of the first 12 received QPSK Symbols
figure();
subplot(211);
stem(real(data(1:12)));
title('The real part of the data-vector');
axis([0 12 -1.5 1.5]);
set(gca,'ytick',[-1 1]);
xlabel('Symbol number');
ylabel('Value');
subplot(212);
stem(imag(data(1:12)));
title('The imaginary part of the data-vector');
axis([0 12 -1.5 1.5]);
set(gca,'ytick',[-1 1]);
xlabel('Symbol number');
ylabel('Value');

%Plot of the first 24 received bits and the soundOut
if strcmp(data_type, 'sound');
figure();
subplot(211)
stem(DataOut(1:24));
title('The DataOut-vector');
axis([0 24 -0.5 1.5]);
set(gca,'ytick',[0 1]);
xlabel('Bit number');
ylabel('Bit value');
subplot(212)
t = 0:1/Fs_sound:(length(wavData)-1)/Fs_sound;
plot(t, wavData);
title('The received wave-file')
axis([0 0.35 -0.6 0.6]);
xlabel('Time - Seconds')
ylabel('Amplitude')    
else
figure();
stem(DataOut(1:24));
title('The DataOut-vector');
axis([0 24 -0.5 1.5]);
set(gca,'ytick',[0 1]);
xlabel('Bit number');
ylabel('Bit value');    
end

end
