function s_tot = OFDMtransmitter(symbollength, bandwidth, guard, data_type, numbofcarriers, numberofsymbols, plots)

B = bandwidth;                      %Bandwidth
N = numbofcarriers;                 %Number of carriers                        
Tu = symbollength;                  %Symbol length
freq_space_factor = round(B*Tu/N);  %Factor spacing of the frequency spectrum     
FS=192e3;                           %Sampling freq
T = 1/FS;                           %Elementary period
fc = 38e3;                          %Carrier freq
guard = guard;                      %Guard time
guard_samples = round(guard/T);     %Guard time in samples
tt = 0:T:Tu;                        %Time vector
t_guard = 0:T:(Tu+guard)  ;         %Time vector including guard time
M = numberofsymbols;                %Number of OFDM symbols - reference symbol
f = (1:FS);                         %Frequency vector
ttotal= 0:T:(Tu*(M+1)+M*T+(M)*guard); %Total time including delay

%TRANSMISSION

%Generating binary data
if strcmp(data_type, 'sound');
    [Data1 DataIn Fs_sound bits] = loadSound('Sound.wav', M*N*2, plots); %Generates data based on a wave file
elseif strcmp(data_type, 'random');
    rand('state', 0);
    DataIn = round(rand(2*N*M,1));                                           %Generates random data
end

%Generating QPSK data
Data_QPSK = QPSKmodulator(DataIn, N, M);

%Seral to parallell data
Data_par = zeros(N,M);
k = 1;
m = 1;
for nn = 1:length(Data_QPSK);
    Data_par(k,m) = Data_QPSK(nn);
    if k == N;
        k = 1;
        m = m + 1;
    else
        k = k + 1;
    end    
end

%DPSK
%Creating a random reference symbol
rand('state',1)
bb = -1+2*round(rand(1,N)).'+i*(-1+2*round(rand(1,N))).';

%Mapping the symbols as a phasedifference
phase_shift = zeros(N, M);
Data_phase_diff = zeros(N, M+1);
Data_phase_diff(1:N,1) = bb./sqrt(2);       %Normalize the reference           
dim = size(Data_phase_diff);

for pp = 2:dim(2);
    for qq = 1:dim(1)
       if Data_par(qq,pp-1) == 1+i;
           phase_shift(qq,pp-1) = -1;
       elseif Data_par(qq,pp-1) == -1-i;
           phase_shift(qq,pp-1) = 1;
       elseif Data_par(qq,pp-1) == -1+i;
           phase_shift(qq,pp-1) = i;
       elseif Data_par(qq,pp-1) == 1-i;
           phase_shift(qq,pp-1) = -i;
       end
    %creating a vector where the phasedifference is b-vector
    Data_phase_diff(qq,pp) =(1/abs(phase_shift(qq,pp-1)))*phase_shift(qq,pp-1) * (1/abs(Data_phase_diff(qq,pp-1)))*Data_phase_diff(qq,pp-1);
    end
end

%Zero padding
A = size(Data_phase_diff);
info = zeros(length(tt), M+1);

for zz = 1:M+1
info(1:freq_space_factor:(freq_space_factor*(N/2)),zz) = [Data_phase_diff(1:N/2,zz)];
info(length(tt)-(freq_space_factor*(N/2)-1):freq_space_factor:length(tt),zz) = [Data_phase_diff((N/2+1):A(1),zz)];  %Zeros placed in the middle of the data
end

%Creating M+1 OFDM symbols
for symbol = 1:M+1;

%OFDM
carriers = FS.*ifft(info(1:length(info),symbol));

%Upshifting
s_tilde = carriers.*exp(i*2*pi*fc*tt');

%Inserting guard
s_guard = zeros(1, length(t_guard));
s_guard((guard_samples+1):length(s_guard)) = s_tilde;
%Creating the total signal
s_tot((1+(symbol-1)*length(s_guard)):length(s_guard)*symbol) = s_guard;

%Saving the reference symbol
if symbol==1;
   ref = real(s_tilde); 
end

end

 %s = real(s_tot(guard_samples+1:length(s_tot)));
s = s_tot(guard_samples+1:length(s_tot));

% %Creating wave file
% text = ['signal_',data_type, '_', num2str(N),'_',num2str(M),'_',num2str(guard),'.wav'];
% 
% MAX_SN = max(abs(s_tot));
% wavwrite(s/MAX_SN, FS, 'signal');
% %wavwrite(s/MAX_SN, FS, text);


%Saving variables
if strcmp(data_type, 'sound');
    save('OFDMspace.mat', 'ref', 'Data_par', 'DataIn', 'N', 'M', 'B', 'FS', 'fc', 'Fs_sound', 'bits', 's', 'T', 'Tu', 'f', 'guard', 'guard_samples', 'data_type', 'plots', 'freq_space_factor');
elseif strcmp(data_type, 'random');
    save('OFDMspace.mat', 'ref', 'Data_par', 'DataIn', 'N', 'M', 'B', 'FS', 'fc', 's', 'T', 'Tu', 'f', 'guard', 'guard_samples', 'data_type', 'plots', 'freq_space_factor');
end

if plots == 1

%Plots

if strcmp(data_type, 'sound');
%Plott of soundIn and the binary stream
t = 0:1/Fs_sound:(length(Data1)-1)/Fs_sound;
figure();
subplot(211)
plot(t, (Data1-(2^(bits-1)))/(2^(bits-1)));
title('Wave-file to be transmitted')
axis([0 0.35 -0.6 0.6]);
xlabel('Time - Seconds')
ylabel('Amplitude')
subplot(212)
stem(DataIn(1:24));
title('Bit stream DataIn ');
axis([0 24 -0.5 1.5]);
set(gca,'ytick',[0 1]);
xlabel('Bit number');
ylabel('Bit value');
else
figure()
stem(DataIn(1:24));
title('Bit stream DataIn ');
axis([0 24 -0.5 1.5]);
set(gca,'ytick',[0 1]);
xlabel('Bit number');
ylabel('Bit value');        
end

%Plot of the QPSK symbols
figure();
subplot(211);
stem(real(Data_QPSK(1:12)));
title('The real part of the Data\_QPSK-vector');
axis([0 12 -1.5 1.5]);
set(gca,'ytick',[-1 1]);
xlabel('Symbol number');
ylabel('Value');
subplot(212);
stem(imag(Data_QPSK(1:12)));
title('The imaginary part of the Data\_QPSK-vector');
axis([0 12 -1.5 1.5]);
set(gca,'ytick',[-1 1]);
xlabel('Symbol number');
ylabel('Value');

%Plot of the QPSK symbols
figure();
plot(Data_QPSK,'LineStyle','none', 'Marker','x', 'MarkerSize',10);
title('QPSK symbol constallation');
axis([-1.5 1.5 -1.5 1.5]);
grid on;
xlabel('Real');
ylabel('Imag');

%Plot of the received phase shifts before decision
figure();
plot(phase_shift(1:N,1),'LineStyle','none', 'Marker','x', 'MarkerSize',10);
title('Phase difference');
axis([-1.5 1.5 -1.5 1.5]);
grid on;
xlabel('Real');
ylabel('Imag');

%Plot of the baseband spectrum
figure();
subplot(211);
plot(f/1000, abs(fft(carriers, FS))/FS);
title('Spectrum of the carriers-vector');
axis([0 192 0 3]);
xlabel('Frequency - kHz');
ylabel('Amplitude');
subplot(212);
pwelch(carriers,[],[],[],FS);

%Plot of the frequency spectrum of s(t)
figure();
subplot(211);
plot(f/1000, abs(fft(real(s), FS))/FS);
title('Spectrum of s(t)');
axis([35 41 0 3]);
xlabel('Frequency - kHz');
ylabel('Amplitude');
subplot(212);
pwelch(real(s),[],[],[],FS);
axis([35 41 -50 20]);

end
