function sn_delay = channelMod(channel_delay, SNR, phase_error, impulseresponse)

load OFDMspace.mat;

delay_samples = round(channel_delay/T);

ttotal= 0:T:(Tu*(M+1)+M*T+T*delay_samples+(M)*guard); %Total time including delay

%CHANNEL MODELING
%phase error
theta = zeros(1,length(ttotal));

if phase_error > 0
%Linear phase error    
theta = 0:(phase_error)/(length(s)):(phase_error);
end



s_phase_error = s.*exp(i.*-theta(1:length(s)));
s_phase_error_real = real(s_phase_error);

%Adding white gausian noise
S_0 = mean(s_phase_error_real.^2);

if SNR ~= -1
sigma = S_0/(10^(SNR/10));
else
sigma = 0;
SNR = inf;
end

AWGN = sigma*randn(size(s_phase_error_real));

sn = s_phase_error_real + AWGN;

%Adding delay
sn_delay = zeros(1,length(sn)+delay_samples);
sn_delay((delay_samples+1):length(sn_delay)) = sn;

%Multipaths
if impulseresponse == 1
    
    impulselength = 20e-3;
    timpulse = 0:T:impulselength;

    impresp = zeros(1, length(timpulse));

    impresp(1) = 1;   
    impresp(200) = 0.6;
    impresp(400) = 0.4;
    impresp(600) = -0.3;
    impresp(800) = 0.25;
    impresp(1000) = 0.2;
    impresp(1200) = -0.6;
    impresp(1400) = 0.4;
    impresp(1600) = -0.3;
    impresp(1800) = 0.6;
    impresp(2000) = 0.4;
    impresp(2200) = 0.3;
    impresp(2400) = 0.6;
    impresp(2600) = -0.4;
    impresp(2800) = 0.3;
    impresp(3000) = -0.25;
    impresp(3200) = 0.2;
    

    s_conv = conv(sn_delay, impresp);
    t_conv = 0:T:(impulselength+ttotal(length(ttotal)));

    
    %Creating wave file
    MAX_SN = max(abs(s_conv));
    wavwrite(s_conv/MAX_SN, FS, 'signal');
    REF = ref/MAX_SN;

else
    
    %Creating wave file
    text = ['signal_',data_type, '_', num2str(Tu), '_', num2str(N),'_',num2str(M),'_',num2str(guard),'.wav'];

    MAX_SN = max(abs(sn));
    wavwrite(sn_delay/MAX_SN, FS, 'signal');
%     wavwrite(sn_delay/MAX_SN, FS, text);
    REF = ref/MAX_SN;
end

% %Creating wave file
% text = ['signal_',data_type, '_', num2str(N),'_',num2str(M),'_',num2str(guard),'.wav'];
% 
% MAX_SN = max(abs(s_conv));
% % MAX_SN = max(abs(sn));
% wavwrite(s_conv/MAX_SN, FS, 'signal');
% % wavwrite(sn_delay/MAX_SN, FS, 'signal');
% %wavwrite(sn_delay/MAX_SN, FS, text);
% REF = ref/MAX_SN;

save('MAX_SN.mat', 'MAX_SN', 'SNR', 'phase_error', 'impulseresponse', 'channel_delay');

if plots ==1

%Plots

%Plot of the frequency error
figure()
plot(ttotal, theta(1:length(ttotal)))
title(['Phase error as a function of time']);
axis([0 5.2 0 10]);
set(gca,'ytick',[0 3.1415 6.2831],  'YTickLabel',{'0 ','pi ','2pi '});
xlabel('Time - Seconds');
ylabel('Phase error');



%Plot of the frequency spectrum of the received signal
figure();
subplot(211);
plot(f/1000, abs(fft(sn, FS))/FS);
title(['Spectrum of the received signal']);
axis([35 41 0 3]);
xlabel('Frequency - kHz');
ylabel('Amplitude');
subplot(212);
pwelch(sn,[],[],[],FS);
axis([35 41 -50 20]);

%Plot of the transmitted signal
figure();
subplot(211)
plot(ttotal, sn_delay);
title(['The transmitted signal']);
axis([0 5.2 -600 600]);
xlabel('Time - Seconds');
ylabel('Amplitude');
subplot(212)
plot(ttotal, sn_delay);
axis([1.748 1.76 -400 400]);
xlabel('Time - Seconds');
ylabel('Amplitude');


if impulseresponse == 1
%Plot of the impulseresponse of the channel
figure()
stem(timpulse, impresp);
title(['Channel impulse response']);
xlabel('Time - Seconds');
ylabel('Amplitude');


%Plot of the received signal
figure()
subplot(211)
plot(ttotal, s);
title(['The transmitted signal']);
axis([2.18 2.21 -200 200]);
xlabel('Time - Seconds');
ylabel('Amplitude');
subplot(212)
plot(t_conv, s_conv)
title(['The received signal']);
axis([2.18 2.21 -200 200]);
xlabel('Time - Seconds');
ylabel('Amplitude');
end

end